<?php
/*
	Plugin Name: WooCommerce MoIP Gateway
	Plugin URI: http://woothemes.com/woocommerce
	Description: Integração MoIP para WooCommerce <a href="http://vibemidia.com/woocommerce" title="instructions of use" target="_blank">instruções de uso</a>.
	Version: 1.0.6
	Author: John-Henrique
	Author URI: http://vibemidia.com
	Requires at least: 3.1
	Tested up to: 3.3
	Text domain: woocommerce-gateway-moip
*/

/*  Copyright 2011 John-Henrique 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Plugin updates
 **/
if (is_admin()) {
	
	if ( ! class_exists( 'WooThemes_Plugin_Updater' ) ) require_once( 'woo-updater/plugin-updater.class.php' );
	
	$woo_plugin_updater_moip = new WooThemes_Plugin_Updater( __FILE__ );
	$woo_plugin_updater_moip->api_key = '03cf293d090e8fb6bf71d5f7ec190168';
	$woo_plugin_updater_moip->init();
}

add_action('plugins_loaded', 'woocomerce_gateway_moip', 0);

function woocomerce_gateway_moip() {

	if ( ! class_exists( 'WC_Payment_Gateway' ) ) return;

	add_filter('woocommerce_payment_gateways', 'add_moip_gateway' );

	class WC_Gateway_Moip extends WC_Payment_Gateway {

		public function __construct() {
			global $woocommerce;

			$this->id   = 'moip';
			$this->method_title = 'MoIP';
			$this->icon   = WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/moip-p.png';
			$this->has_fields  = false;
			$this->liveurl   = 'https://www.moip.com.br';
			$this->testurl   = 'https://desenvolvedor.moip.com.br/sandbox';
			$this->url    = $this->liveurl;

			// Load the form fields.
			$this->init_form_fields();

			// Load the settings.
			$this->init_settings();

			// Define user set variables
			$this->title   = $this->settings['title'];
			$this->description  = $this->settings['description'];
			$this->account   = $this->settings['account'];
			$this->token   = $this->settings['token'];
			$this->key   = $this->settings['key'];

			$this->debug  = $this->settings['debug'];
			$this->testmode  = $this->settings['testmode'];

			// Logs
			if ($this->debug=='yes') $this->log = $woocommerce->logger();

			// Enabled?
			if ( $this->testmode == 'yes' ) $this->url = $this->testurl;

			// Valid?
			if ( !$this->is_valid_currency() || !$this->are_credentials_set() )
				$this->enabled = false;


			// Actions
			add_action('init',        array(&$this, 'notificacao' ));
			add_action('woocommerce_receipt_moip', array(&$this, 'receipt_page' ));
			add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
			add_action('admin_notices', array(&$this, 'check'));

		}

		/**
		 * Initialise Gateway Settings Form Fields
		 */
		function init_form_fields() {

			$this->form_fields = array(
				'enabled' => array(
					'title' => __( 'Enable/Disable', 'woocommerce-gateway-moip' ),
					'type' => 'checkbox',
					'label' => __( 'Enable MoIP standard', 'woocommerce-gateway-moip' ),
					'default' => 'yes'
				),
				'title' => array(
					'title' => __( 'Title', 'woocommerce-gateway-moip' ),
					'type' => 'text',
					'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce-gateway-moip' ),
					'default' => __( 'moip', 'woocommerce-gateway-moip' )
				),
				'description' => array(
					'title' => __( 'Description', 'woocommerce-gateway-moip' ),
					'type' => 'textarea',
					'description' => __( 'This controls the description which the user sees during checkout.', 'woocommerce-gateway-moip' ),
					'default' => __("Pay via MoIP you can pay with your credit card, billet and eletronic transfer same you no have a MoIP account.", 'woocommerce-gateway-moip')
				),
				'account' => array(
					'title' => __( 'MoIP Account', 'woocommerce-gateway-moip' ),
					'type' => 'text',
					'description' => __( 'Please enter your MoIP account; this is needed in order to take payment.', 'woocommerce-gateway-moip' ),
					'default' => ''
				),
				'token' => array(
					'title' => __( 'MoIP Token', 'woocommerce-gateway-moip' ),
					'type' => 'text',
					'description' => sprintf(__( 'Please enter your MoIP token; this is needed in order to take payment. See %sToken of security%s', 'woocommerce-gateway-moip' ), '<a href="http://labs.moip.com.br/api/" target="_blank">', '</a>'),
					'default' => ''
				),
				'key' => array(
					'title' => __( 'MoIP Key', 'woocommerce-gateway-moip' ),
					'type' => 'text',
					'description' => __( 'Please enter your MoIP Key; this is needed in order to take payment.', 'woocommerce-gateway-moip' ),
					'default' => ''
				),
				'testmode' => array(
					'title' => __( 'MoIP sandbox', 'woocommerce-gateway-moip' ),
					'type' => 'checkbox',
					'description' => __( 'Enable MoIP sandbox', 'woocommerce-gateway-moip' ),
					'default' => 'yes'
				),
				'debug' => array(
					'title' => __( 'Enable/Disable Log', 'woocommerce-gateway-moip' ),
					'type' => 'checkbox',
					'description' => __( 'Enable logging (<code>woocommerce/logs/moip.txt</code>)', 'woocommerce-gateway-moip' ),
					'default' => ''
				),

			);

		} // End init_form_fields()

		/**
		 * Admin Panel Options
		 * - Options for bits like 'title' and availability on a country-by-country basis
		 *
		 * @since 1.0.0
		 */
		public function admin_options() {

?>
	    	<h3><?php echo __('MoIP Gateway', 'woocommerce-gateway-moip'); ?></h3>
	    	<p><?php echo __('MoIP works by sending the user to MoIP Gateway to enter their payment information.', 'woocommerce-gateway-moip');?>
	    		<?php echo sprintf(__('This Payment Gateway works only accounts Seller or Professional. See %s documentation%s', 'woocommerce-gateway-moip'), '<a href="http://labs.moip.com.br/api/" target="_blank">', '</a>'); ?></p>
	    	<table class="form-table">
	    	<?php
			$this->check();


			// Generate the HTML For the settings form.
			$this->generate_settings_html();
?>
			</table><!--/.form-table-->
	    	<?php
		} // End admin_options()

		public function check() {

			if ( !$this->is_valid_currency() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'woocommerce-gateway-moip' ) ."</strong>: ". __( 'MoIP does not support your store\'s currency. You need to select the currency of Brazil Real.', 'woocommerce-gateway-moip' ) ."</p>
				</div>";
			endif;

			if ( ! $this->are_credentials_set() ) :
				echo "<div class=\"inline error\">
					<p><strong>". __( 'Gateway Disabled', 'woocommerce-gateway-moip' ) ."</strong>: ". __( 'You must give the token of your account email.', 'woocommerce-gateway-moip' ) ."</p>
				</div>";
			endif;
		}


		/**
		 * There are no payment fields for moip, but we want to show the description if set.
		 **/
		function payment_fields() {
			if ($this->description) echo wpautop(wptexturize($this->description));
		}

		/**
		 * Check if MoIP can be used with the store's currency.
		 * For now only work with Real of Brazil, but...
		 *
		 * @since 1.0
		 */
		function is_valid_currency() {
			if (get_option('woocommerce_currency') == "BRL" ) {
				return true;
			}


			return false;
		}

		/**
		 * Check if MoIP Credentials are set
		 *
		 * @since 1.0
		 */
		function are_credentials_set() {
			if ( empty( $this->account ) || empty( $this->token ) )
				return false;
			else
				return true;
		}

		/**
		 * Generate the MoIP button link
		 **/
		public function generate_moip_form( $order_id ) {
			global $woocommerce;
			$order = new WC_Order( $order_id );

			$woocommerce->add_inline_js('
			jQuery(function(){
				jQuery("body").block(
					{
						message: "<img src=\"'.esc_url( $woocommerce->plugin_url() ).'/assets/images/ajax-loader.gif\" alt=\"'. __("Redirecting...", 'woocommerce-gateway-moip') .'\" style=\"float:left; margin-right: 10px;\" />'.__('Thank you for your order. We are now redirecting you to MoIP to make payment.', 'woocommerce-gateway-moip').'",
						overlayCSS:
						{
							background: "#fff",
							opacity: 0.6
						},
						css: {
					        padding:        20,
					        textAlign:      "center",
					        color:          "#555",
					        border:         "3px solid #aaa",
					        backgroundColor:"#fff",
					        cursor:         "wait",
					        lineHeight:		"32px"
					    }
					});
				jQuery("#submit_moip_payment_form").click();
			});');

			// HTML to payment
			$html = '<form action="'.esc_url( "https://www.moip.com.br/PagamentoMoIP.do" ).'" method="post" id="moip_payment_form">
						'. $this->form_html( $order ) .'
						<input type="submit" class="button-alt" id="submit_moip_payment_form" value="'.__('Pay via moip', 'woocommerce-gateway-moip').'" /> <a class="button cancel" href="'.esc_url( $order->get_cancel_order_url() ).'">'.__('Cancel order &amp; restore cart', 'woocommerce-gateway-moip').'</a>
					</form>';

			return $html;

		}

		/**
		 * Process the payment and return the result
		 * Executa no checkout 'finalizar compra -> pagar'
		 *
		 **/
		function process_payment( $order_id = 0 ) {
			global $woocommerce;

			$order = new WC_Order( $order_id );

			return array(
				'result'  => 'success',
				'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(get_option('woocommerce_pay_page_id'))))
			);

		}

		/**
		 * receipt_page
		 **/
		function receipt_page( $order ) {
			echo '<p>'.__('Thank you for your order, please click the button below to pay with moip.', 'woocommerce-gateway-moip').'</p>';
			echo $this->generate_moip_form( $order );
		}

		/**
		 * Ajuda a identificar o status de pagamento
		 * @see https://www.moip.com.br/AdmTransactionManual.do
		 * @param Integer $status status de pagamento retornado
		 * @return Array status de pagamento convertido para o WooCommerce
		 */
		public function status_helper( $status = null, $order = null ) {

			switch ( $status ) {

			case 1:
				$arrStatus['status'] = "completed"; // paga
				$arrStatus['log'] = __('Manual confirmation of payment, Check payment confirmation', 'woocommerce-gateway-moip');

				// change payment status
				$order->payment_complete();

				break;

			case 3:
				$arrStatus['status'] = "pending"; // Boleto impresso
				$arrStatus['log'] = __('Printed billet, Waiting payment confirmation', 'woocommerce-gateway-moip'); // aguardando pagamento

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 4:// disponivel
				$arrStatus['status'] = "completed"; // disponivel
				$arrStatus['log'] = __('Payment completed', 'woocommerce-gateway-moip');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 5: // disputa
				$arrStatus['status'] = "on-hold";
				$arrStatus['log'] = __('Failed on process of negociation (dispute)', 'woocommerce-gateway-moip');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 6:
				$arrStatus['status'] = "pending"; // aguardando
				$arrStatus['log'] = __('Payment in analysis, Waiting payment confirmation', 'woocommerce-gateway-moip');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 7:
				$arrStatus['status'] = "refunded"; // devolvida
				$arrStatus['log'] = __('Payment refunded', 'woocommerce-gateway-moip');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

				break;

			case 5:
				$arrStatus['status'] = "cancelled"; // cancelada
				$arrStatus['log'] = __('Order cancelled', 'woocommerce-gateway-moip');

				// cancel this order
				$order->cancel_order( $order->id );

				break;

				// improvavel mas nao custa prevenir
			default:
				$arrStatus['status'] = 'pending';
				$arrStatus['log'] = __('Payment pending, Waiting payment confirmation', 'woocommerce-gateway-moip');

				// add note to control
				$order->add_order_note( $arrStatus['log'] );

			}

			return $arrStatus;
		}

		function notificacao() {
			global $woocommerce;

			// who are you?
			if ( isset( $_POST['cod_moip'] ) && isset( $_POST['forma_pagamento'] ) ) {

				// short cut
				$order_code  = $_POST['cod_moip'];
				$order_ID   = $_POST['id_transacao'];
				$order_status  = $_POST['status_pagamento'];
				$order_total  = $_POST['valor'];
				$paymentMethod  = $_POST['forma_pagamento'];
				$paymentType  = $_POST['tipo_pagamento'];
				$consumerEmail  = $_POST['email_consumidor'];


				// this order exists?
				$pedido = new woocommerce_order( (int) $order_ID );


				/**
				 * For security reasons we check if the application
				 * exists and that the total order value is the
				 * same as the amount charged in store
				 *
				 * if any errors occur
				 * kill this process
				 */
				if ( !is_int( $pedido->id ) && $pedido->order_total != $order_total ) {
					$woocommerce->add_error(__('Oops, there was an error during the payment process, please try again.', 'woocommerce-gateway-moip'));
					header("HTTP/1.0 404 Not Found");
					exit;
				}else {

					// what is this?
					$arrStatus = $this->status_helper( $order_status, $pedido );
					wp_redirect( $this->get_return_url( $order ) );
					header("HTTP/1.0 200 OK");
				}
			}
		}

		public function percorre_produtos( $order ) {
			$arrXml = array();

			// Cart Contents
			$item_loop = 0;
			if (sizeof($order->items)>0) : foreach ($order->items as $item) :
					if ($item['qty']) :

						$item_loop++;

					$item_name = $item['name'];

				if (isset($item['item_meta'])) :
					if ($meta = woocommerce_get_formatted_variation( $item['item_meta'], true )) :
						$item_name .= ' ('.$meta.')';
					endif;
				endif;

			$arrXml[] = "
		            <item>
		                <id>". $item_loop ."</id>
		                <description>". $item_name ."</description>
		                <amount>". number_format( $item['cost'], 2, '.', '' ) ."</amount>
		                <quantity>". $item['qty'] ."</quantity>
		                <weight>0</weight>
		            </item>
		            ";

			endif;
			endforeach;

			// shortcut
			$amount = $order->order_shipping + $order->order_tax + $order->order_shipping_tax;

			// delivery free no need send
			if ( !empty( $amount ) ) {
				// delivery costs
				$arrXml[] = "
	            <item>
	                <id>frete</id>
	                <description>". __("Delivery via", 'woocommerce-gateway-moip' ) ." ". $order->shipping_method_title ."</description>
	                <amount>". number_format( $order->order_shipping + $order->order_tax + $order->order_shipping_tax, 2, '.', '' ) ."</amount>
	                <quantity>1</quantity>
	                <weight>0</weight>
	            </item>
	            ";
			}

			endif;

			return $arrXml;
		}


		/**
		 * HTML form to simple integration
		 *
		 * @see https://www.moip.com.br/AdmCheckout.do?method=manual
		 * @param Object $order
		 */
		public function form_html( $order ) {

			// isn't Brazil?
			if ( !$this->is_valid_currency() ) {
				return;
			}

			$order->billing_phone = str_replace(array('(', '-', ' ', ')'), '', $order->billing_phone);
			$ddd = substr($order->billing_phone, 0, 2 );
			$telefone = substr($order->billing_phone, 2 );

			//$urlReturn = $this->get_return_url( $order ); // Gateway no have support to Query Strings
			// no support to query string
			$urlReturn = get_permalink(get_option('woocommerce_thanks_page_id'))  .'?order='. $order->id;

			$item_loop = 1;

			$html  = '
				<!-- Store -->
				<input name="id_carteira" type="hidden" value="'. $this->account .'" />
				<input name="id_transacao" type="hidden" value="'. $order->id .'" />
				<input name="descricao" type="hidden" value="Pagamento do pedido Nº '. $order->id .'" />
				<input name="url_retorno" type="hidden" value="'. $urlReturn .'" />

				<!-- Products -->
				<input name="nome" type="hidden" value="Pagamento do pedido '. $order->id .'" />
				<input name="valor" type="hidden" value="'. str_replace( '.', '', $order->order_total ) .'" />

				<!-- Customer -->
				<input name="pagador_nome" type="hidden" value="'. $order->billing_first_name ." ". $order->billing_last_name .'" />
				<input name="pagador_email" type="hidden" value="'. $order->billing_email .'" />
				<input name="pagador_pais" type="hidden" value="Brasil" />
				';

			return $html;
		}


		/**
		 * Request a token of Gateway
		 * @see http://labs.moip.com.br/pdfs/Integra%C3%A7%C3%A3o%20API%20-%20Pagamento%20%C3%9Anico.pdf
		 * @param Object $order
		 * @return String
		 */
		public function request_token( $order ) {
			global $woocommerce;

			$url = $this->get_return_url( $order );

			$base = $this->token .":". $this->key;
			$auth = base64_encode($base);
			$header[] = "Authorization: Basic " . $auth;
			$header[] = "Content-Type: application/xml; charset=UTF-8";
			$param = '
			<EnviarInstrucao>
				<InstrucaoUnica>
					<Razao>Pagamento do pedido '. $order->id .'</Razao>
					<IdProprio>'. $order->id . $this->id_proprio .'</IdProprio>
					<URLNotificacao>'. $url .'</URLNotificacao>
					<URLRetorno>'. $url.'</URLRetorno>

					<Recebedor>
						<LoginMoIP>'. $this->account .'</LoginMoIP>
						<!-- Unnecessary
						<Email></Email>
						-->
					</Recebedor>

					<!-- coming soon
					<FormasPagamento>
					     <FormaPagamento>BoletoBancario</FormaPagamento>
					     <FormaPagamento>CarteiraMoIP</FormaPagamento>
					     <FormaPagamento>CartaoCredito</FormaPagamento>
					     <FormaPagamento>DebitoBancario</FormaPagamento>
					     <FormaPagamento>FinanciamentoBancario</FormaPagamento>
					</FormasPagamento>
					-->
					<Valores>
					     <Valor moeda="BRL">'. number_format( $order->order_total, 2, '.', '' ) .'</Valor>
					</Valores>

					<Pagador>
				        <Nome>'. $order->billing_first_name ." ". $order->billing_last_name .'</Nome>
				        <Email>'. $order->billing_email .'</Email>
				        <EnderecoCobranca>
			                <Pais>BRA</Pais>
			                <TelefoneFixo>'. $order->billing_phone .'</TelefoneFixo>
				        </EnderecoCobranca>
					</Pagador>

				</InstrucaoUnica>
			</EnviarInstrucao>
			';

			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, $this->url ."/ws/alpha/EnviarInstrucao/Unica" );
			curl_setopt($curl, CURLOPT_HTTPHEADER, $header);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_USERAGENT, "Mozilla/4.0");
			curl_setopt($curl, CURLOPT_POST, true);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $param);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			$curlReturn = curl_exec($curl);
			curl_close($curl);

			$xml= simplexml_load_string($curlReturn);

			if ( $xml->Resposta->Status != "Sucesso" || !isset( $xml->Resposta->Token ) ) {

				$woocommerce->add_error(__('Oops, there was an error during the payment process, unauthorized.', 'woocommerce-gateway-moip'));
				$return = false;
			}else {

				// shortcut
				$token = $xml->Resposta->Token;


				$return = $this->url .'/Instrucao.do?token='. $token;

				/**
				 * Write token to accept pay in other time
				 * this is necessary why in MoIP for each
				 * request added a intention of payment
				 * unique by order.
				 */
				add_post_meta( $order->id, 'moip_token', (string) $return, true );
			}

			// request again but change ID
			return $return;
		}

	}

	load_plugin_textdomain('woocommerce-gateway-moip', false, basename( dirname( __FILE__ ) ) . '/languages' );

}

function add_moip_gateway( $methods ) {
	$methods[] = 'WC_Gateway_Moip'; return $methods;
}
